/*
 * Este fichero contiene funciones básicas de consulta/introducción SQL y cálculo
 * relacionadas con la introducción de las áreas de cobertura de los satélites.
 */ 
var mysql = require('mysql');

// Creamos una conexión a la BBDD con los siguientes parámetros y credenciales.
var satinfoDB = mysql.createConnection(
    {
        host    :   'localhost',
        user    :   'consultor',
        password:   'contrasena',
        database:   'SatInfo'
    }    
);
satinfoDB.connect();

/* Ejemplo de uso
var orden = "INSERT INTO Canal VALUES ('11200','17024','Da Vinci Learning','10758','V','Nagravision 3','T','HD',null,'27500','3/4','DVB-S2','QPSK');";
satinfoDB.query(orden, function(err, rows, fields){
    if(err) throw err;
    
    console.log("Exectuted 'SELECT * FROM Canal' ");
    console.log(rows);
    satinfoDB.end();
});
*/

exports.getHuellasDeTranspondedor = function(callback){
    /*
     * Obtenemos la identificación de las huellas que hay consideradas en la tabla Transpondedor.
     * Aquí están todas, ya que esta tabla se rellena a partir del fichero csv de nuestro
     * proveedor de información de tv satelital.
     */
    var orden = 'SELECT DISTINCT posicion, nombre_huella FROM Transpondedor;';
    satinfoDB.query(orden, function(err, rows, fields){
        if(err) throw err;
        
        console.log("getHuellasDeTranspondedor/rows: "+JSON.stringify(rows));
        callback(rows);
    });
}

exports.getHuellasDeHuella = function(callback){
    /*
     * Obtenemos la identificación de las huellas que hay guardadas en la tabla Huella,
     * que son aquellas de las cuales ya conocemos las coordenadas que determinan el
     * contorno de las mismas.
     */
    var orden = 'SELECT posicion, nombre_huella FROM Huella;';
    satinfoDB.query(orden, function(err, rows, fields){
        if(err) throw err;
        
        console.log("getHuellasDeHuella/rows: "+JSON.stringify(rows));
        callback(rows);
    });
}

exports.compararListasDeHuellas = function(deTranspondedor, deHuella){
    /*
     * Efectuamos la intersección de deTranspondedor y de deHuella, devolviendo el resto.
     */
    console.log("compararListasDeHuellas()");
    
    // Definimos la variable de respuesta
    var huellasPendientes = [];
    
    // Si deHuella está vacía, la respuesta es deTranspondedor al completo, no hace falta procesar
    if(deHuella == ''){
        return deTranspondedor;
    }else{
        // Recorremos deTranspondedor y partimos de que la huella en cuestión está pendiente
        for(var n = 0; n <= deTranspondedor.length-1; n++){
            var estaPendiente = true;
            
            // Recorremos deHuella para comparar cada elemento con el elemento de deTranspondedor que estamos considerando en la iteración
            for(var i = 0; i <= deHuella.length-1; i++){
                //console.log("# deTranspondedor[0].posicion: "+deTranspondedor[n].posicion);
                //console.log("# deHuella[0].posicion: "+deHuella[i].posicion);
                //console.log("# deTranspondedor[0].nombre_huella: "+deTranspondedor[n].nombre_huella);
                //console.log("# deHuella[0].nombre_huella: "+deHuella[i].nombre_huella);
                
                // Si un elemento de deHuella coincide en posicion y nombre_huella con el elemento de deTranspondedor en cuestión, esa huella ya está dibujada
                if((deHuella[i].posicion == deTranspondedor[n].posicion)&&(deHuella[i].nombre_huella == deTranspondedor[n].nombre_huella)){
                    estaPendiente = false;
                }
            }
            
            // Cuando una huella está pendiente de ser dibujada, se añade su identificación a la variable de respuesta
            if(estaPendiente){
                huellasPendientes.push(deTranspondedor[n]);
            }
        }
        return huellasPendientes;
    }
}

exports.reordenarObjetos = function(preOrden){
    /*
     * Esta función toma un objeto en preOrden con forma {{"posicion":"PA","nombre_huella":"N1"},{"posicion":"PA","nombre_huella":"N1"}}
     * y le da forma {{"posicion":"PA", "beams":["N1", "N2"]}, {"posicion":"PB", "beams":["N3","N4"]}}
     */
    
    // Definimos la variable de respuesta
    var postOrden = [];
    
    // posicionesProcesadas es un array auxiliar que, como crece a la vez que 
    // huellasPendientesOrganizadas, nos permite obtener rapidamente el 
    // indice de una posicion orbital determinada. 
    var posicionesProcesadas = [];
    
    // Recorremos preOrden...
    for (var i = 0; i <= preOrden.length-1; i++){
        // ... extrayendo su primer elemento y guardándolo en la variable "elemento"
        var elemento = preOrden.shift();
        
        // Si el atributo posicion de elemento no consta todavía en posicionesProcesadas...
        if(posicionesProcesadas.indexOf(elemento.posicion) === -1){
            // ... añadimos la posición y el nombre de la huella a la variable de respuesta...
            postOrden.push({"posicion": elemento.posicion, "beams": [elemento.nombre_huella]});
            // ... y la posicion al array auxiliar posicionesProcesadas
            posicionesProcesadas.push(elemento.posicion);
        }else{
            // Si la posicion de elemento ya existe en posicionesProcesadas añadimos el nombre_huella al array "beams"
            // del elemento correspondiente en postOrden
            postOrden[posicionesProcesadas.indexOf(elemento.posicion)].beams.push(elemento.nombre_huella);
        }
    }
    return postOrden;
}

exports.guardarHuella = function(parametros, callback){
    /*
     * Esta funcion guarda el dibujo indicado por el usuario, con su identificacion (posicion-nombre)
     * en la tabla Huella de la BBDD
     */
    
    // Convertimos el JSON a String para evitar alteraciones
    var huellas = JSON.stringify(parametros.huellas);
    // Construimos la sentencia SQL
    var orden = "INSERT INTO Huella values ('"
                                                +parametros.posicion+"','"
                                                +parametros.nombre_huella+"','"
                                                +huellas+"');";
    //console.log("guardarHuella/orden: "+orden);
    
    // Hacemos efectiva la sentencia SQL
    satinfoDB.query(orden, function(err, result){
        if(err) throw err;
        callback();
    });
}

exports.getDibujoDeHuella = function(parametros, callback){
    /*
     * Esta funcion devuelve el dibujo de la huella indicada por los parametros.
     */
    
    // Construimos la sentencia SQL
    var orden = "SELECT huella FROM Huella WHERE posicion='"+parametros.posicion+"' AND nombre_huella='"+parametros.nombre_huella+"';";
    // Hacemos efectiva la sentencia SQL
    satinfoDB.query(orden, function(err, result){
        if(err) throw err;
        var dibujo = JSON.parse(result[0].huella);
        callback(dibujo);
    });
}

exports.eliminarDibujo = function(parametros, callback){
    /*
     * Esta funcion elimina el dibujo de la huella identificada por los parametros
     * que hay almacenado en la BBDD, para perimitir volver a trazarlo.
     */
    
    // Construimos la sentencia SQL...
    var orden = "DELETE FROM Huella WHERE posicion ='"+parametros.posicion+"' AND nombre_huella='"+parametros.nombre_huella+"';";
    // ...y la hacemos efectiva
    satinfoDB.query(orden, function(err, result){
        if(err) throw err;
        
        console.log(result);
        callback(result);
    });
}