/*
 * En este archivo encontramos las grandes funciones que conforman el programa.
 * Cada una de ellas tiene un reflejo en server.js que deriva las peticiones a 
 * estas funciones. El sentido de este archivo es estructurar las llamadas a las
 * funciones básicas de cálculo y consulta a la base de datos que se encuentran
 * en UtilidadesCNL.js y UtilidadesHLL.js
 */
var UtilidadesCNL = require('./UtilidadesCNL.js');
var UtilidadesHLL = require('./UtilidadesHLL.js');
//------------------------------------------------------------------------------
exports.getIdiomasDisponibles = function(callback){
    /*
     * Esta función sólo hace de puente entre el servidor y UtilidadesCNL que tiene acceso a la BBDD.
     */
    console.log("getIdiomasDisponibles()");
    UtilidadesCNL.getIdiomasDisponibles(function(idiomasDisponibles){
        callback(idiomasDisponibles);
    });
}
exports.obtenerCanalesDisponibles = function(datosDeConsulta, callback){
    /*
     * Esta función responde con los canales y la información relevante de ellos disponibles durante
     * la ruta proporcionada por el usuario.
     */
    console.log("Recibido de cliente: "+JSON.stringify(datosDeConsulta));
    
    // Nos quedamos sólo con los transpondedores que ofrecen algún idioma de los solicitados por el usuario
    UtilidadesCNL.filtrarTranspondersPorIdioma(datosDeConsulta.idiomas, function(transpondersPotencialmenteUtiles){
        //console.log("transpondersPotencialmenteUtiles: "+JSON.stringify(transpondersPotencialmenteUtiles));
        
        // Comprobamos que haya algún transpondedor que cumpla el requisito de idiomas antes de continuar con el proceso
        if(transpondersPotencialmenteUtiles[0]){
            // Filtramos los transpondedores que cumplen la condición de idiomas atendiendo a si toda la ruta proporcionada por el usuario
            // está dentro del área de cobertura de ellos. Consideramos la antena utilizada para elegir la huella según el nivel de ganancia necesario
            UtilidadesCNL.filtrarTranspondersPorHuella(datosDeConsulta.ruta, datosDeConsulta.antena, transpondersPotencialmenteUtiles, function(listaTIDfiltrada){
                //console.log("listaTIDfiltrada: "+listaTIDfiltrada);
                
                // Comprobamos que algún transpondedor cumpla el requisito de cobertura antes de continuar con el proceso
                if(listaTIDfiltrada[0]){
                    // Obtenemos la información relevante de los canales que cumplen todos los requisitos para estar disponibles
                    UtilidadesCNL.getInfoDeCanales(listaTIDfiltrada, function(respuesta){
                        //console.log("infoDeCanalesRespuesta: "+JSON.stringify(respuesta));
                        
                        // Devolvemos la información de los canales disponibles
                        callback({"codigo": 000, "mensaje": "Consulta exitosa.", "payload": respuesta});   
                    });
                }else{
                    console.log("Ningún transpondedor cumple los requisitos de cobertura.");
                    callback({"codigo": 003, "mensaje": "Ningún transpondedor cumple los requisitos de cobertura.", "payload": null});
                }
            });
        }else{
            console.log("Ningún transpondedor cumple los requisitos de idioma.");
            callback({"codigo": 002, "mensaje": "Ningún transpondedor cumple los requisitos de idioma.", "payload": null});
        }
        //callback(transpondersPotencialmenteUtiles);
    });
}//obtenerCanalesDisponibles

//------------------------------------------------------------------------------
exports.getHuellas = function(callback){
    /*
     * Esta funcion devuelve los objetos la posición y nombre de las huellas (con esto las identificamos)
     * diferenciando entre aquellas de las que ya tenemos el dibujo y las que están pendientes de dibujar.
     */
    console.log("getHuellas()");
    
    // Llamamos a la función de utilidad que obtiene las posicion-nombre de todas las huellas que hay consideradas en la BBDD (todas están en la tabla Transpondedor)
    UtilidadesHLL.getHuellasDeTranspondedor(function(huellasDeTranspondedor){
        //Llamamos a la funcion que  obtiene las posicion-nombre de las huellas que ya han sido dibujadas (guardadas estas en la tabla Huella)
        UtilidadesHLL.getHuellasDeHuella(function(huellasDeHuella){
            // Aquellas huellas de huellasDeTranspondedor que no están en huellasDeHuella están pendientes de dibujar
            var huellasPendientes = UtilidadesHLL.compararListasDeHuellas(huellasDeTranspondedor, huellasDeHuella);
            //console.log("huellasPendientes: "+JSON.stringify(huellasPendientes));
            
            // Reordenamos los objetos para pasar devolver uno de más fácil interpretación en el cliente
            var huellasPendientesOrganizadas = UtilidadesHLL.reordenarObjetos(huellasPendientes);
            var huellasGuardadas = UtilidadesHLL.reordenarObjetos(huellasDeHuella);
            //console.log("huellasPendientesOrganizadas: "+JSON.stringify(huellasPendientesOrganizadas));
            
            callback(huellasPendientesOrganizadas, huellasGuardadas);
            //callback({"pendientes": huellasPendientesOrganizadas, "guardadas": huellasGuardadas});
        });
    });
}

exports.crearHuella = function(parametros, callback){
    /*
     * Esta función sólo hace de puente entre el servidor y UtilidadesHLL que tiene acceso a la BBDD.
     */
    UtilidadesHLL.guardarHuella(parametros, function(){
        callback({"codigo": 000, "mensaje": "Huella guardada exitosamente.", "payload": {"TID": parametros.TID}});
    });
}

exports.getDibujo = function(parametros, callback){
    /*
     * Esta función sólo hace de puente entre el servidor y UtilidadesHLL que tiene acceso a la BBDD.
     */
    UtilidadesHLL.getDibujoDeHuella(parametros, function(dib){
        callback(dib);
    });
}

exports.borrarDibujo = function(parametros, callback){
    /*
     * Esta función sólo hace de puente entre el servidor y UtilidadesHLL que tiene acceso a la BBDD.
     */
    UtilidadesHLL.eliminarDibujo(parametros, function(result){
        callback(result);
    });
}